@@ -16,19 +16,19 @@
 # along with gnome-tweak-tool.  If not, see <http://www.gnu.org/licenses/>.
 
 import os.path
-import shutil
 import zipfile
 import tempfile
 import logging
+import json
 
 from gi.repository import Gtk
 from gi.repository import GLib
 
-from gtweak.utils import walk_directories
+from gtweak.utils import walk_directories, extract_zip_file
 from gtweak.gsettings import GSettingsSetting
 from gtweak.gshellwrapper import GnomeShell
 from gtweak.tweakmodel import Tweak, TweakGroup
-from gtweak.widgets import GConfComboTweak, GSettingsComboEnumTweak, GSettingsSwitchTweak, build_label_beside_widget, build_horizontal_sizegroup, build_combo_box_text
+from gtweak.widgets import ZipFileChooserButton, GConfComboTweak, GSettingsComboEnumTweak, GSettingsSwitchTweak, build_label_beside_widget, build_horizontal_sizegroup, build_combo_box_text
 
 class ShowWindowButtons(GConfComboTweak):
     def __init__(self, **options):
@@ -41,17 +41,6 @@ class ShowWindowButtons(GConfComboTweak):
             (':minimize,maximize,close', 'All')),
             **options)
 
-class _ThemeZipChooser(Gtk.FileChooserButton):
-    def __init__(self):
-        Gtk.FileChooserButton.__init__(self, title="Select theme file")
-
-        f = Gtk.FileFilter()
-        f.add_mime_type("application/zip")
-        self.set_filter(f)
-
-        #self.set_width_chars(15)
-        self.set_local_only(True)
-
 class ShellThemeTweak(Tweak):
 
     THEME_EXT_NAME = "user-theme@gnome-shell-extensions.gnome.org"
@@ -60,7 +49,7 @@ class ShellThemeTweak(Tweak):
     THEME_DIR = os.path.join(GLib.get_home_dir(), ".themes")
 
     def __init__(self, **options):
-        Tweak.__init__(self, "Shell theme", "Install custom or user themes for gnome-shell", **options)
+        Tweak.__init__(self, "Shell Theme", "Install custom or user themes for gnome-shell", **options)
 
         #check the shell is running and the usertheme extension is present
         error = "Unknown"
@@ -74,7 +63,7 @@ class ShellThemeTweak(Tweak):
                 #check the correct gsettings key is present
                 try:
                     self._settings = GSettingsSetting(ShellThemeTweak.THEME_GSETTINGS_SCHEMA)
-                    name = self._settings.get_value(ShellThemeTweak.THEME_GSETTINGS_NAME)
+                    name = self._settings.get_string(ShellThemeTweak.THEME_GSETTINGS_NAME)
 
                     #assume the usertheme version is that version of the shell which
                     #it most supports (this is a poor assumption)
@@ -98,81 +87,98 @@ class ShellThemeTweak(Tweak):
         else:
             hb = Gtk.HBox()
 
-            #build a combo box with all the valid theme options
-            valid = walk_directories( (ShellThemeTweak.THEME_DIR,), lambda d:
+            #include both system, and user themes
+            #note: the default theme lives in /system/data/dir/gnome-shell/theme
+            #      and not themes/, so add it manually later
+            dirs = [os.path.join(d, "themes") for d in GLib.get_system_data_dirs()]
+            dirs += [ShellThemeTweak.THEME_DIR]
+
+            valid = walk_directories(dirs, lambda d:
                         os.path.exists(os.path.join(d, "gnome-shell")) and \
                         os.path.exists(os.path.join(d, "gnome-shell", "gnome-shell.css")))
-            #manually add Adwiata to represent the default
-            #valid.append( ("Adwiata", "") )
 
+            #build a combo box with all the valid theme options
+            #manually add Adwaita to represent the default
             cb = build_combo_box_text(
                     self._settings.get_string(ShellThemeTweak.THEME_GSETTINGS_NAME),
-                    ("", "Adwiata"),
+                    ("", "Adwaita"),
                     *[(v,v) for v in valid])
             cb.connect('changed', self._on_combo_changed)
             hb.pack_start(cb, False, False, 5)
             self.combo = cb
 
-            chooser = _ThemeZipChooser()
+            chooser = ZipFileChooserButton("Select a theme file")
             chooser.connect("file-set", self._on_file_set)
             hb.pack_start(chooser, False, False, 0)
 
             self.widget = build_label_beside_widget(self.name, hb)
             self.widget_for_size_group = chooser
     
-    def _extract_theme_zip(self, z, theme_name, theme_members_path):
-        tmp = tempfile.mkdtemp()
-        dest = os.path.join(ShellThemeTweak.THEME_DIR, theme_name, "gnome-shell")
-
-        logging.info("Extracting theme %s to %s" % (theme_name, tmp))
-
-        try:
-            if os.path.exists(dest):
-                shutil.rmtree(dest)
-            z.extractall(tmp)
-            shutil.copytree(os.path.join(tmp, theme_members_path), dest)
-            return theme_name
-        except OSError:
-            self.notify_error("Error installing theme")
-            return None
-
     def _on_file_set(self, chooser):
         f = chooser.get_filename()
 
         with zipfile.ZipFile(f, 'r') as z:
             try:
                 fragment = ()
+                theme_name = None
                 for n in z.namelist():
                     if n.endswith("gnome-shell.css"):
                         fragment = n.split("/")[0:-1]
-                        break
+                    if n.endswith("gnome-shell/theme.json"):
+                        logging.info("New style theme detected (theme.json)")
+                        #new style theme - extract the name from the json file
+                        tmp = tempfile.mkdtemp()
+                        z.extract(n, tmp)
+                        with open(os.path.join(tmp,n)) as f:
+                            try:
+                                theme_name = json.load(f)["shell-theme"]["name"]
+                            except:
+                                logging.warning("Invalid theme format", exc_info=True)
 
                 if not fragment:
                     raise Exception("Could not find gnome-shell.css")
 
-                #old style themes name was taken from the zip name
-                if fragment[0] == "theme" and len(fragment) == 1:
-                    theme_name = os.path.basename(f)
-                else:
-                    theme_name = fragment[0]
+                if not theme_name:
+                    logging.info("Old style theme detected (theme.json)")
+                    #old style themes name was taken from the zip name
+                    if fragment[0] == "theme" and len(fragment) == 1:
+                        theme_name = os.path.basename(f)
+                    else:
+                        theme_name = fragment[0]
+
                 theme_members_path = "/".join(fragment)
 
-                installed_name = self._extract_theme_zip(
-                                        z,
-                                        theme_name,
-                                        theme_members_path)
-                if installed_name:
-                    print self.combo.get_model().append( (installed_name, installed_name) )
+                ok, updated = extract_zip_file(
+                                z,
+                                theme_members_path,
+                                os.path.join(ShellThemeTweak.THEME_DIR, theme_name, "gnome-shell"))
+
+                if ok:
+                    if updated:
+                        self.notify_info("%s theme updated successfully" % theme_name)
+                    else:
+                        self.notify_info("%s theme installed successfully" % theme_name)
+
+                    #I suppose I could rely on updated as indicating whether to add the theme
+                    #name to the combo, but just check to see if it is already there
+                    model = self.combo.get_model()
+                    if theme_name not in [r[0] for r in model]:
+                        model.append( (theme_name, theme_name) )
+                else:
+                    self.notify_error("Error installing theme")
+
 
             except:
                 #does not look like a valid theme
                 self.notify_error("Invalid theme file")
+                logging.warning("Error parsing theme zip", exc_info=True)
+
         #set button back to default state
         chooser.unselect_all()
 
     def _on_combo_changed(self, combo):
         val = combo.get_model().get_value(combo.get_active_iter(), 0)
-        self._settings.set_value(ShellThemeTweak.THEME_GSETTINGS_NAME, val)
+        self._settings.set_string(ShellThemeTweak.THEME_GSETTINGS_NAME, val)
 
         #reloading the theme is not really necessary, the user-theme should pick
         #pick up the change.
